#!/usr/bin/env python
# -*- coding: utf-8 -*-

from db_paribhasha import credentials
from db_paribhasha.dynamo import dynamo_helpers as db_helpers
from db_paribhasha.dynamo.run_metadata import RunMetadata
from db_paribhasha.error_handler import OperationNotPerformedError


def create_table():
    """
    Creates table if table doesn't exist
    """

    if RunMetadata.exists():
        raise OperationNotPerformedError("Table already exists")
    RunMetadata.create_table(
        read_capacity_units=credentials.DYNAMO_GENERIC_RW_CAP,
        write_capacity_units=credentials.DYNAMO_GENERIC_RW_CAP)


def get_run_metadata_detail(run_id):
    """
    Returns the detail of a given Run id.

    Parameters
    ----------
    run_id: String/Int
        Run id to get detail

    Returns
    -------
    run_metadata: Pynamo object
        Run metadata object for the given Run ID.
    """
    return RunMetadata.get(int(run_id))


def create_run_metadata(run_id, import_info=None, export_info=None):
    """
    Creates the metadata entry for a given run id

    Parameters
    ----------
    run_id : Int/String
        Run id to create metadata
    import_info : String, optional
        Import info to store (the default is None)
    export_info : String, optional
        Export info to store (the default is None)

    Returns
    -------
    run_id: Int
        Run id for which metadata is created
    """

    run_metadata = RunMetadata(
        run_id=int(run_id), import_info=import_info, export_info=export_info)
    db_helpers.save_dynamo_object(run_metadata)
    return run_id


def update_run_metadata(run_id, import_info=None, export_info=None):
    """
    Updates the run metadata for a given run id

    Parameters
    ----------
    run_id : String/Integer
        Run id to update metadata
    import_info : [String], optional
        [Import info for run] (the default is None)
    export_info : [String], optional
        [Export info for run] (the default is None)

    """
    run_meta = RunMetadata.get(int(run_id))
    if import_info:
        run_meta.import_info = import_info
    if export_info:
        run_meta.export_info = export_info
    db_helpers.save_dynamo_object(run_meta)
