from builtins import str
from db_paribhasha.error_handler import DatabaseEntryError, InvalidInputError
from db_paribhasha import db_constants as db_const
from db_paribhasha.base_models.sql_base_model import session
from db_paribhasha.workflow.iteration import iteration_table

def get_value(iter_id, column_name):
    """
    This function gives value of a given cell in database.

    Parameters
    ----------
    iter_id: Integer
        The primary key of the iteration table.
    coloum_name: String
        Name of the coloumn in the database

    Returns
    -------
    String: Column_value
            The value that is in a given cell of the database
    """
    if column_name not in list(iteration_table.Iteration.__table__.columns.keys()):
        raise DatabaseEntryError(db_const.INVALID_COLUMN)

    if has_iteration(iter_id):
        iter_row = session.query(iteration_table.Iteration).get(iter_id)
        return getattr(iter_row, column_name)
    else:
        raise DatabaseEntryError(db_const.INVALID_ITER_ID)


# TODO - Merge get_value & get_values functions
# doing it this way so that it doesn't break any other API
def get_values(iter_ids, column_name):
    """
    This function gives all the values corresponding to the input list of
    primary key

    Parameters
    ----------
    iter_ids: List of integer
        List of primary keys
    column_name: String
        Name of the column whose values needs to be returned

    Returns
    -------
    column_values: List of dict
        List of dictionary with Primary key as key and column value as value
        of dict
    """

    if column_name not in list(iteration_table.Iteration.__table__.columns.keys()):
        raise DatabaseEntryError(db_const.INVALID_COLUMN)
    iter_rows = \
        session.query(iteration_table.Iteration).filter(
            iteration_table.Iteration.id.in_(iter_ids)).all()

    column_values = []
    for iter_row in iter_rows:
        column_val = {}
        column_val["iter_id"] = iter_row.id
        column_val[column_name] = getattr(iter_row, column_name)
        column_values.append(column_val)
    
    get_ids = [column['iter_id'] for column in column_values]
    
    if not set(get_ids) == set(iter_ids):
        raise DatabaseEntryError(db_const.INVALID_ITER_ID)

    return column_values

def has_iteration(iter_id):
    """
    Checks if a given iteration id is present in the table
    or not.

    Parameters
    ------------
    iter_id
        The primary key of the row in iteration table

    Returns
    -------------
     bool
        which tells the existance
    """
    iter_row = session.query(iteration_table.Iteration).get(iter_id)
    if iter_row is not None:
        return True
    else:
        return False


# TODO - similar function exists in iteration_resource_utils
def get_iteration_type(iter_id):
    """
    Returns the iteration type of a given iteration id.

    Parameters
    -----------
    iter_id: Integer
        Iteration id to check for parent

    Returns
    -------
    String: parent
        - 'component' if it is run
        - 'workflow' if it is workflow request
    """
    if not has_iteration(iter_id):
        raise DatabaseEntryError(db_const.INVALID_ITER_ID)
    iter_row = session.query(iteration_table.Iteration).get(iter_id)
    return getattr(iter_row, 'type')


def get_workflow_id(iter_id):
    """
    Returns the iteration's parent id, ie. a workflow request or a run id for a
    given iteration id.

    Parameters
    -----------
    iter_id: Integer
        Iteration id to check for parent

    Returns
    -------
    String (JSON): run/workflow_request id
    """
    iter_row = session.query(iteration_table.Iteration).get(iter_id)
    parent_id = str(getattr(iter_row, 'workflow_request'))
    parent_type = db_const.TYPE_WORKFLOW
    return_dict = {
        'type': parent_type,
        'id': parent_id
    }
    return return_dict


def is_workflow(iter_id):
    """
    Checks if a given iteration id is associated to a workflow or not.

    Parameters
    ------------
    iter_id
        The primary key of the row in iteration table

    Returns
    -------------
     bool
        which specifies whether the iteration is linked to a workflow or not
    """
    iter_type = get_iteration_type(iter_id)
    if iter_type == db_const.TYPE_WORKFLOW:
        return True
    else:
        return False


def is_workflow_batch(iter_ids):
    """
    Checks if each iteration id in the given list is associated to a
    workflow or not.

    Parameters
    ------------
    iter_ids
        List of the primary key of the row in iteration table

    Returns
    -------------
    list of bool
        which specifies whether the iteration is linked to a workflow or not
    """
    iter_types_dict = get_values(iter_ids, 'type')

    iter_types = [iter_type_dict["type"] for iter_type_dict in iter_types_dict]    

    if len(set(iter_types)) > 1:
        raise InvalidInputError(db_const.INVALID_ITER_TYPE)

    if iter_types[0] == db_const.TYPE_WORKFLOW:
        return True
    return False


def get_iteration_list(condition_args, ascending=False):
    """
    function returns the list of iterations which satisfy the condition 
    passed as the argument.

    Parameters
    ----------
    condition_args: dict
        dictionary which contains the where conditions of the sql query.

    Returns
    --------
    iter_dict_list: list
        list of dictionaries. Each dictionary contains the information of
        a particular iteration.

    """
    condition_args['is_active'] = 'true'
    if ascending:
        iterations = session.query(iteration_table.Iteration).filter_by(**condition_args).\
            order_by(iteration_table.Iteration.id.asc()).all()
    else:
        iterations = session.query(iteration_table.Iteration).filter_by(**condition_args).\
            order_by(iteration_table.Iteration.id.desc()).all()

    return [itr.as_dict() for itr in iterations]
